#!/bin/sh
BSP_MODEL=$2

doOptional=false
useDebianSource=false

fromearly=false
fromdevelop=false
domerge=false

. ./build-env.sh

export ARCH=arm64
if [ "X${BSP_ARCH}" = "Xarmhf" ]; then
  export ARCH=arm
  export CROSS_COMPILE=arm-linux-gnueabihf-

elif [ "X${BSP_ARCH}" = "Xarm64" ]; then
  export ARCH=arm64
  export CROSS_COMPILE=aarch64-linux-gnu-

elif [ "X${BSP_ARCH}" = "Xriscv64" ]; then
  export ARCH=riscv
  export CROSS_COMPILE=riscv64-linux-gnu-

  gcver=9.3.1
  gcnam=riscv64-glibc-gcc-thead_20200903
  gctar=${gcnam}-${gcver}-`uname -m`.tar.gz
  if [ ! -e `pwd`/${gcnam} ]; then
    if [ ! -e ${gctar} ]; then
      wget -N https://seafile.servator.de/sbc/nezha/toolchain/${gctar}
    fi
    if [ -e ${gctar} ]; then
      mkdir `pwd`/${gcnam}
      cd `pwd`/${gcnam}
      tar xzf ../${gctar}
      cd ..
    fi
  fi
  if [ "X`uname -m`" = "Xx86_64" -a ! -e `pwd`/prebuilt -a ! -e `pwd`/${gcnam} ]; then
    git clone https://dl.linux-sunxi.org/D1/SDK/projects/prebuilt.git prebuilt
  fi

  t=`pwd`/prebuilt/gcc/linux-x86/riscv/toolchain-thead-glibc/riscv64-glibc-gcc-thead_20200702/bin/riscv64-unknown-linux-gnu-
  u=`pwd`/${gcnam}/${gcver}/bin/riscv64-unknown-linux-gnu-
  if [ -e ${u}gcc ]; then
    export CROSS_COMPILE=$u
  elif [ -e ${t}gcc ]; then
    export CROSS_COMPILE=$t
  fi

elif [ "X${BSP_ARCH}" = "Xamd64" ]; then
  export ARCH=x86
  export CROSS_COMPILE=x86_64-linux-gnu-

elif [ "X${BSP_ARCH}" = "Xi686" ]; then
  export ARCH=x86
  export CROSS_COMPILE=i686-linux-gnu-
elif [ "X${BSP_ARCH}" = "Xpowerpc" ]; then
  export ARCH=powerpc
  export CROSS_COMPILE=powerpc-linux-gnu-
fi

lxmmmkb=`echo ${lxdebkb} | cut -d '.' -f 1-2`

if [ ! -e android_bootimg_tools ]; then
  git clone https://github.com/scpcom/android_bootimg_tools android_bootimg_tools
fi

if [ ! -e linux-${lxmmmkb}.y ]; then
  #git clone -b linux-${lxmmmkb}.y https://git.kernel.org/pub/scm/linux/kernel/git/stable/linux-stable.git linux-${lxmmmkb}.y
  #git clone --depth 100 https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git linux-${lxmmmkb}.y
  git clone -b mainline-${lxmmmkb} https://github.com/scpcom/linux linux-${lxmmmkb}.y
fi

echo "Copying files from linux-${lxmmmkb}.y git..."
mkdir linux && \
cp -apP linux-${lxmmmkb}.y/. linux/ && \
cd linux

VERSION=`cat Makefile | grep -E '^VERSION.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
PATCHLEVEL=`cat Makefile | grep PATCHLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`
SUBVERSION=`cat Makefile | grep SUBLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`

quiltpatches() {
  if [ -e ../patches/$2 ]; then
    mkdir -p patches/$1

    for p in `cat ../patches/$2 | grep -v '^#' | grep patch` ; do
      b=`basename $p`

      if [ -e ../patches/${p} ]; then
        cp -p ../patches/${p} patches/${p}
      else
        cp -p  ../patches/${b} patches/${p}
      fi
    done

    cat ../patches/$2 >> patches/series

    quilt push -a
  fi

}

simplepatch() {
  p=$1
  f=$2

  b=`basename $p`
  d=`dirname $p`
  l=${p}
  n=`echo $b | sed s/\.patch$//g`

  mkdir -p patches/${d}

  if [ -e ../patches/${p} ]; then
    cp -p ../patches/${p} patches/${p}
  else
    cp -p  ../patches/${b} patches/${p}
  fi

  if echo $n | grep -q -E '^mali-' ; then
    if [ -e drivers/gpu/arm/mali400 ]; then
      sed -i "s|/src/devicedrv/|/drivers/gpu/arm/mali400/|g" patches/${p}
    else
      sed -i "s|/src/devicedrv/|/drivers/gpu/arm/|g" patches/${p}
    fi
  fi

  if echo $n | grep -q -E '^linux-' ; then
    n=`echo $n | sed s/^linux-4\.1.\.y-//g`
    n=`echo $n | sed s/^linux-4\.4\.y-//g`
    n=`echo $n | sed s/^linux-4\.1.-//g`

  elif echo $n | grep -q -E '^mali-' ; then
    n=`echo $n | sed s/^mali-r.p.-//g`
  fi

  git apply patches/${p}
  git add ${f}
  git commit -m "${d} ${n}"
  echo "${p}" >> patches/series
}

simplegitam() {
  s=../patches
  p=$1

  mkdir -p patches

  #git am --keep-cr < ../patches/${p}

      nogitam=1
      gitamcont=0

      if grep -m1 -q "^From ........................................ Mon Sep 17 00:00:00 2001" ${s}/${p} ; then
        c=`grep -m1 "^From ........................................ Mon Sep 17 00:00:00 2001" ${s}/${p} | sed s/'^From '//g | cut -d ' ' -f1`

        #sed -i -e "0,/^---$/ s/^---$/\n${d} ${c}\n---/" ${s}/${p}

        if git am --keep-cr < ${s}/${p} ; then
          nogitam=0
          echo ${p} >> patches/series-am
        else
          gitamcont=1
          echo ""
          echo "${s}/${p} failed"
          exit 1
        fi
      else
        echo "${p}: wrong patch type"
        exit 1
      fi

      if [ $nogitam = 1 ]; then
        if ! git apply --stat --apply ${s}/${p} ; then
          patch -p1 --no-backup-if-mismatch < ${s}/${p}
          echo ${p} >> patches/series-no-git
        else
          echo ${p} >> patches/series-no-am
        fi

        git add .

        if [ $gitamcont = 1 ]; then
          git am --continue
        else
          echo "${p}: manual patch without git am"
          exit 1
        fi
      fi

  echo "${p}" >> patches/series
}

gitpatches() {
  if [ -e ../patches/$2 ]; then
    echo ""
    echo "Apply series patches/$2"
    echo ""

    mkdir -p patches/$1

    l=none
    for p in `cat ../patches/$2 | grep -v '^#' | grep patch` ; do
      b=`basename $p`
      d=`dirname $p`
      l=${p}
      n=`echo $b | sed s/\.patch$//g`

      if [ -e ../patches/${p} ]; then
        cp -p ../patches/${p} patches/${p}
      else
        cp -p  ../patches/${b} patches/${p}
      fi

      echo "Apply patch patches/${p}"

      #if grep -q '\\r' patches/${p} ; then
      #  sed -i -e 's/\r$//g' patches/${p}
      #fi

      if echo $n | grep -q -E '^mali-' ; then
        if [ -e drivers/gpu/arm/mali400 ]; then
          sed -i "s|/src/devicedrv/|/drivers/gpu/arm/mali400/|g" patches/${p}
        else
          sed -i "s|/src/devicedrv/|/drivers/gpu/arm/|g" patches/${p}
        fi
      fi

      nogitcp=1
      gitcpcont=0
      nogitam=1
      gitamcont=0
      if false ; then
      if grep -m1 -q "^From ${n} Mon Sep 17 00:00:00 2001" patches/${p} ; then
        rc=0
        git cherry-pick ${n} || rc=$?
        echo "git cherry-pick ${n}: rc=${rc}"
        if [ $rc = 0 ] ; then
          nogitcp=0
          echo ${p} >> patches/series-cherry-pick
        elif [ $rc != 128 ] ; then
          #gitcpcont=1
          git cherry-pick --abort
        fi
      fi
      fi

      if [ "${d}" = "." ] ; then
        d=$1
      fi

      if [ $nogitcp = 1 -a $gitcpcont = 0 ]; then
      if grep -m1 -q "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} ; then
        c=`grep -m1 "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} | sed s/'^From '//g | cut -d ' ' -f1`

        sed -i -e "0,/^---$/ s/^---$/\n${d} ${c}\n---/" patches/${p}

        if git am --keep-cr < patches/${p} ; then
          nogitam=0
          echo ${p} >> patches/series-am
        else
          gitamcont=1
          #git am --skip
        fi
      fi
      fi

      if [ $nogitam = 1 -a $nogitcp = 1 ]; then
        pb=`echo ${p} | sed s/'\.patch$'/'-bin.patch'/g`

        #echo "Check patches/${pb}"
        if [ -e ../patches/${pb} ]; then
          cp -p ../patches/${pb} patches/${pb}
          echo git apply --stat --apply patches/${pb}
          git apply --stat --apply patches/${pb}
        fi

        if ! git apply --stat --apply patches/${p} ; then
          #git apply --stat --apply --3way patches/${p}
          #sed -i -e 's/\r$//g' patches/${p}
          patch -p1 --no-backup-if-mismatch < patches/${p}
          echo ${p} >> patches/series-no-git
        else
          echo ${p} >> patches/series-no-am
        fi
      fi

      if echo $n | grep -q -E '^linux-' ; then
        n=`echo $n | sed s/^linux-4\.1.\.y-//g`
        n=`echo $n | sed s/^linux-4\.4\.y-//g`
        n=`echo $n | sed s/^linux-4\.1.-//g`

      elif echo $n | grep -q -E '^mali-' ; then
        n=`echo $n | sed s/^mali-r.p.-//g`

      elif echo $n | grep -q -E '^0...-' ; then
        n=`echo $n | sed s/^0...-//g`

      elif echo $n | grep -q -E '^0....-' ; then
        n=`echo $n | sed s/^0....-//g`
      elif echo $n | grep -q -E '^1....-' ; then
        n=`echo $n | sed s/^1....-//g`

      elif grep -m1 -q "^From ${n} Mon Sep 17 00:00:00 2001" patches/${p} ; then
        if ! grep -q '^---$' patches/${p} ; then
          exit 1
        fi

      else
        exit 1
      fi

      if [ $nogitam = 1 -a $nogitcp = 1 ]; then
        echo git commit -m "${d} ${n}"
        git add .
        #git commit -m "${d} ${n}"
        #echo "${d} ${n}" > patches/${p}.msg

        if grep -m1 -q "^From ........................................ Mon Sep 17 00:00:00 2001" patches/${p} ; then
          if ! grep -q '^---$' patches/${p} ; then
            exit 1
          fi
          cat patches/${p} | grep -m1 -B399 '^---$' | sed /^---$/d | grep -m1 -A399 '^Subject:' | sed s/'^Subject: '/''/g > patches/${p}.msg
        else
          echo "${d} ${n}" > patches/${p}.msg
        fi
        
        if [ $gitcpcont = 1 ]; then
          git cherry-pick --continue
        elif [ $gitamcont = 1 ]; then
          git am --continue
        else
          git commit -F patches/${p}.msg
        fi
      fi

      echo ""
    done

    echo "Now in patch patches/${l}"
    cat ../patches/$2 >> patches/series

    #quilt push -a
  fi

}

applypatches() {
  gitpatches $1 $2
  #quiltpatches $1 $2
}

gitmerge() {
  mname=$1
  mbranch=$2

  start_range=`git rev-list --simplify-by-decoration -2 HEAD`

  start_range_0=`echo $start_range | cut -d ' ' -f 1`
  start_range_1=`echo $start_range | cut -d ' ' -f 2`

  #git merge -s recursive -X ours -X renormalize -X no-renames origin/${mbranch} | tee ../merge-${mname}.log
  git merge -s recursive -X ours -X renormalize -X no-renames ${mbranch} | tee ../merge-${mname}.log

  commit_range=`git rev-list --simplify-by-decoration -2 HEAD`

  commit_range_0=`echo $commit_range | cut -d ' ' -f 1`
  commit_range_1=$start_range_0

  git log --decorate=no --reverse ${commit_range_1}..${commit_range_0} | grep -E '^commit ' | cut -d ' ' -f 2- > ../merge-${mname}-commit-list.txt

  cm=`cat ../merge-${mname}-commit-list.txt | tail -1`

  for c in `cat ../merge-${mname}-commit-list.txt` ; do
    if [ ${c} != ${cm} ] ; then
      echo "${mname}/${c}.patch" >> patches/series-${mname}-merge
    fi
    if [ ${mname} = mainline ]; then
      grep ${c} ../patches/series-${mname}-??? | cut -d ':' -f 2- >> patches/series-${mname}-quilt
    else
      grep ${c} ../patches/series-${mname}-20?????? | cut -d ':' -f 2- >> patches/series-${mname}-quilt
    fi
  done

  cat patches/series-${mname}-merge >> patches/series-merge
  cat patches/series-${mname}-merge >> patches/series
}

lxmergever=${lxmmmkb}
#git checkout -b mainline-${lxmergever}

if [ ! -e drivers/gpu/arm/midgard -a ! -e drivers/gpu/arm/midgard_for_linux -a ! -e ../mali-midgard_for_linux-rockchip.tar.gz ]; then
  git checkout develop-4.4
  git checkout 27f039b43ada8d0301867505ce9a91d8b5c604bc
  tar czf ../mali-mali400-rockchip.tar.gz drivers/gpu/arm/mali400
  tar czf ../mali-mali400-platform-rockchip.tar.gz drivers/gpu/arm/mali400/mali/platform/rk
  tar czf ../mali-bifrost-rockchip.tar.gz drivers/gpu/arm/bifrost
  tar czf ../mali-bifrost_for_linux-rockchip.tar.gz drivers/gpu/arm/bifrost_for_linux
  tar czf ../mali-midgard-rockchip.tar.gz drivers/gpu/arm/midgard
  tar czf ../mali-midgard_for_linux-rockchip.tar.gz drivers/gpu/arm/midgard_for_linux
fi

if $fromdevelop ; then
  git checkout mainline-${lxmmmkb}
  git checkout odroid-${lxmmmkb}.y

  git checkout -b tempsbc-${lxmmmkb}.y
else
  #git checkout sbc-${lxmmmkb}-rc1
  if [ $BSP_GROUP = nezha -o $BSP_GROUP = rocksbc ]; then
    git checkout ${BSP_GROUP}-${lxmmmkb}.y
  elif [ $BSP_GROUP = gfiber ]; then
    git checkout ${BSP_GROUP}-${lxmmmkb}.y-mindspeed
  elif ! git checkout sbc-${lxmmmkb}.y ; then
    #git checkout ls1024a-${lxmmmkb}.y
    if ! git checkout mainline-${lxmmmkb} ; then
      git checkout linux-${lxmmmkb}.y
    fi

    #git checkout df2445bf77833674ebf790d2e6fcfd1d389b8a7b # Bluetooth: Add a new 13d3:3496 QCA_ROME device
    #git checkout ffd831a3fc0bebacfbd0f4735345e4c350291e38 # Merge tag 'imx-maintainers-4.18' of git://git.kernel.org/pub/scm/linux/kernel/git/shawnguo/linux into next/soc
    #git checkout fd129f8941cf2309def29b5c8a23b62faff0c9d0 # Merge git://git.kernel.org/pub/scm/linux/kernel/git/bpf/bpf-next
    #git checkout c1144d29f405ce1f4e6ede6482beb3d0d09750c6 # Merge branch 'for-4.18/alps' into for-linu

    #git checkout 11e7c21880617b8383ad8ad059ae9a07e5abe68e # Merge tag 'for-linus-20180605' of git://git.kernel.dk/linux-block
    #git checkout 3c89adb0d11117f64d5b501730be7fb2bf53a479 # Merge tag 'pm-4.18-rc1' of git://git.kernel.org/pub/scm/linux/kernel/git/rafael/linux-pm
    #git checkout 5231804cf9e584f3e7e763a0d6d2fffe011c1bce # Merge tag 'leds_for_4.18-rc1' of git://git.kernel.org/pub/scm/linux/kernel/git/j.anaszewski/linux-leds
    #git checkout f60342fac9fae20ada2cd5faadbc2a1337cae03f # Merge tag 'mmc-v4.18' of git://git.kernel.org/pub/scm/linux/kernel/git/ulfh/mmc
    #git checkout 07c4dd3435aa387d3b58f4e941dc516513f14507 # Merge tag 'usb-4.18-rc1' of git://git.kernel.org/pub/scm/linux/kernel/git/gregkh/usb

    #git checkout abf7dba7c4f77d781f6df50fefb19a64c5dc331f # Merge tag 'char-misc-4.18-rc1' of git://git.kernel.org/pub/scm/linux/kernel/git/gregkh/char-misc
    #git checkout a22e48cf317d22a4326dc19e906e6b5f4f92e94e # Merge tag 'tty-4.18-rc1' of git://git.kernel.org/pub/scm/linux/kernel/git/gregkh/tty
    #git checkout 0bbddb8cbe7a8765e9c6ef598a33b50461934f88 # Merge branch 'for-4.18' of git://git.kernel.org/pub/scm/linux/kernel/git/tj/libata
    #pcie sata ok with extra patches #git checkout 0ad39cb3d70fb4324d127aeceee7f63e3f71605c # Merge tag 'kconfig-v4.18' of git://git.kernel.org/pub/scm/linux/kernel/git/masahiroy/linux-kbuild
    #git checkout 285767604576148fc1be7fcd112e4a90eb0d6ad2 # Merge tag 'overflow-v4.18-rc1' of git://git.kernel.org/pub/scm/linux/kernel/git/kees/linux
    #git checkout 1c8c5a9d38f607c0b6fd12c91cbe1a4418762a21 # Merge git://git.kernel.org/pub/scm/linux/kernel/git/davem/net-next
    #git checkout fdea70d26a471e002f2afc3a48821323b699f1e6 # Merge branch 'for-linus' of git://git.kernel.org/pub/scm/linux/kernel/git/jikos/hid
    #pcie sata ok with extra patches #git checkout 7d3bf613e99abbd96ac7b90ee3694a246c975021 # Merge tag 'libnvdimm-for-4.18' of git://git.kernel.org/pub/scm/linux/kernel/git/nvdimm/nvdimm
    #git checkout 883cad5ba8cc2d9b740b4ad0a8a91063c99c75a3 # Merge tag 'mfd-next-4.18' of git://git.kernel.org/pub/scm/linux/kernel/git/lee/mfd
    #git checkout 3477accaece59199484c4b7586c88235e265f3d0 # Merge branch 'opp/linux-next' of git://git.kernel.org/pub/scm/linux/kernel/git/vireshk/pm
    #git checkout 2ca4eb85744b1cd93076418ac8430ba527cc35bb # Merge branch 'dccp-fixes-around-rx_tstamp_last_feedback'
    #git checkout 896a349228c494aa65279ccfc41d2395898f6f4d # Merge branch 'fixes-v4.18-rc3' of git://git.kernel.org/pub/scm/linux/kernel/git/jmorris/linux-security
    #git checkout 8979319f2d361b5729b215e1d47cb5bbcaca9d76 # Merge tag 'pci-v4.18-fixes-2' of git://git.kernel.org/pub/scm/linux/kernel/git/helgaas/pci
    #git checkout 28c20cc73b9cc4288c86c2a3fc62af4087de4b19 # Merge tag 'drm-fixes-2018-07-20' of git://anongit.freedesktop.org/drm/drm
    #git checkout 1a3d8691fd2ae95f5443b2d57f91f28f6ff4700d # Merge tag 'pci-v4.18-fixes-4' of git://git.kernel.org/pub/scm/linux/kernel/git/helgaas/pci
    #git checkout 6b4703768268d09ac928c64474fd686adf4574f9 # Merge branch 'fixes' of git://git.armlinux.org.uk/~rmk/linux-arm

    #git checkout v${lxmmmkb}
    #git checkout v${lxmmmkb}.222
    #git checkout v${lxmmmkb}.232
    #git checkout v${lxmmmkb}.238
    #git checkout v${lxmmmkb}.237
    #git checkout v${lxmmmkb}.236
    #git checkout v${lxmmmkb}.240
    #git checkout v${lxmmmkb}.255
    #git checkout v${lxmmmkb}.256
    #git checkout v${lxmmmkb}.263
    git checkout -b sbc-${lxmmmkb}.y
  fi

  #git checkout ${lxmmmkb}.207-2975-sbc
  #git checkout ls1024a-${lxmmmkb}.y
  #git checkout 150651225de3cbed2e3431b89201068ad901db95
  #git checkout -b nas5xx-${lxmmmkb}.y

  #git checkout odroidgoA-${lxmmmkb}.y
  #git checkout rocksbc-${lxmmmkb}.y
  #git checkout sbc-android-${lxmmmkb}.y
  #git checkout linux-${lxmmmkb}.y
  #git checkout v${lxmmmkb}

  #git checkout nezha-${lxmmmkb}.y
  #git checkout 5.4.61-2260-nezha
  #git cherry-pick 3c6a8e2f334e22f46475214c45420660b77e94b3
  #git checkout 5.4.159-2350-nezha
  #git checkout 5.4.167-2560-sbc
  #git checkout 5.4.167-2510-sbc

  #git checkout 83e56ecb56c18252f0acd184b5ac043c6bfadb0a
  #git checkout mainline-${lxmmmkb}
  #git checkout v${lxmmmkb}.58
  #git checkout ${lxmmmkb}.134-3100-sbc
  #git checkout -b tst-${lxmmmkb}.y
  #git checkout ${lxmmmkb}.147-3230-sbc
  #git checkout ${lxmmmkb}.73-4290-sbc 
  #git checkout ${lxmmmkb}.73-4330-sbc
fi

VERSION=`cat Makefile | grep -E '^VERSION.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
PATCHLEVEL=`cat Makefile | grep PATCHLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`
SUBVERSION=`cat Makefile | grep SUBLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`

if $fromearly ; then
mkdir -p patches
mkdir -p patches/platform

for f in `cat ../patches/series-odroidc2 | grep -E '^platform/' | cut -d / -f 2-` ; do
  cp -p ../patches/${f} patches/platform/
done

cat ../patches/series-odroidc2 >> patches/series

if [ $doOptional = true ]; then
  for f in `cat ../patches/series-odroidc2-optional | grep -E '^platform/' | cut -d / -f 2-` ; do
    cp -p ../patches/${f} patches/platform/
  done

  cat ../patches/series-odroidc2-optional >> patches/series
fi

cp -p ../patches/stmmac_platform_mac_addr_set.patch patches/
echo "stmmac_platform_mac_addr_set.patch" >> patches/series

quilt push -a

if [ $doOptional = true ]; then
  for f in `cat ../patches/series-odroidc2-binary` ; do
    git apply ../patches/${f}
  done
fi
fi

if $fromdevelop ; then
applypatches platform series-odroidc2

simplepatch \
 platform/linux-5.4.y-stmmac_platform_mac_addr_set.patch \
 drivers/net/ethernet/stmicro/stmmac/stmmac_platform.c
fi

#git merge -s recursive -X ours -X renormalize -X no-renames mainline-${lxmergever} | tee ../merge.log

if [ -e ../patches/mainline ]; then
  #mv patches/platform/mainline patches/
  for s in `ls ../patches/series-mainline-???` ; do
    b=`basename $s`
    v=`basename $s | cut -d '-' -f 3`
    if [ $SUBVERSION -lt $v ]; then
      echo ""
      if [ -e ../patches/${b}-prepare ]; then
        applypatches mainline ${b}-prepare
      fi
      #applypatches mainline ${b}
      #echo ""
      #echo "" >> patches/series
      #cat $s >> patches/series
      #quilt push -a
    fi
  done
fi

if $domerge ; then
gitmerge mainline mainline-${lxmergever}
fi

if [ -e ../patches/mainline ]; then
  #mv patches/platform/mainline patches/
  for s in `ls ../patches/series-mainline-???` ; do
    b=`basename $s`
    v=`basename $s | cut -d '-' -f 3`
    #echo "$SUBVERSION -lt $v"
    if [ $SUBVERSION -lt $v ]; then
      echo ""
      if [ -e ../patches/${b}-post ]; then
        applypatches mainline ${b}-post
      fi
      #echo ""
      #echo "" >> patches/series
      #cat $s >> patches/series
      #quilt push -a
    fi
  done
fi

if $fromdevelop ; then
git revert --no-edit 8360063bfae53a765464aa39755b49cf736f1df0

applypatches platform series-cpuinfo

if [ -e drivers/gpu/arm ]; then
  simplepatch \
    r7p0/mali-r6p2-2008-umplock_fix_build.patch \
    drivers/gpu/arm
fi

if [ -e drivers/gpu/arm ]; then
  simplepatch \
    r7p0/mali-r6p2-2009-fix_5.4_build.patch \
    drivers/gpu/arm
fi

if [ -e drivers/gpu/arm ]; then
  simplepatch \
    r7p0/mali-r6p2-2012-mali_dt_broken.patch \
    drivers/gpu/arm
fi

simplepatch \
  platform/linux-5.4.y-odroid-n1-add_rk3399_restart_driver.patch \
  drivers/power/reset

simplegitam linux-5.4.y-0001-arm64-dts-rockchip-ODROID-N1-import.patch
simplegitam linux-5.4.y-0002-arm64-dts-rockchip-ODROID-N1-remove-unused-nodes.patch
simplegitam linux-5.4.y-0003-arm64-dts-rockchip-ODROID-N1-use-pin-names.patch
simplegitam linux-5.4.y-0004-arm64-dts-rockchip-ODROID-N1-rename-pins-and-regulat.patch
simplegitam linux-5.4.y-0005-arm64-dts-rockchip-ODROID-N1-update.patch

simplegitam 1b280ec8eb7e3a8d7c1d9a8f35782cdcc146963d.patch

simplepatch \
  platform/linux-5.4.y-add_missing_wake_lock_api_wrapper_functions.patch \
  include/linux/wakelock.h

simplepatch \
  platform/linux-5.4.y-0006-arm64-dts-rockchip-ODROID-N1-add_missing_mali_lines.patch \
  arch/arm64/boot/dts/rockchip/rk3399-odroidn1-linux.dts

simplegitam linux-5.4.y-0007-arm64-dts-rockchip-ODROID-N1-assign-i2c3-to-hdmi.patch

simplegitam linux-5.4.y-drm-rockchip-Add-32MHz-pixel-clock.patch

if [ -e drivers/gpu/arm ]; then
simplegitam mali-r6p2-2013-add-mali450-rockchip-r7p0-platform.patch
simplegitam mali-r6p2-2014-modify-mali450-rockchip-r7p0-build.patch
simplegitam mali-r6p2-2015-fix-5.4-build.patch

simplegitam mali-midgard-r14p0-add-midgard-meson-r14p0-platform.patch
simplegitam mali-midgard-r14p0-midgard-meson-r14p0-disable-UMP.patch
simplegitam mali-midgard-r14p0-midgard-meson-r14p0-fix-5.4-build.patch
fi

simplegitam linux-5.4.y-drm-meson-dw-hdmi-wait_for_edid.patch

simplegitam linux-5.4.y-odroid-n1-rk3399-restart-set_system_rev.patch
simplegitam linux-5.4.y-soc-rockchip-add_cpuinfo_support.patch
simplegitam linux-5.4.y-soc-rockchip-cpuinfo-add-of_platform_h.patch

simplegitam linux-5.4.y-arm64-dts-rockchip-rk3399-add-gpu_power_model.patch
simplegitam linux-5.4.y-arm64-dts-rockchip-ODROID-N1-moved-gpu_power_model.patch
simplegitam linux-5.4.y-arm64-dts-rockchip-rockpro64-update.patch

simplegitam linux-5.4.y-arm64-dts-rockchip-rockpro64-add-bootargs-cpuinfo-an.patch

if [ -e drivers/gpu/arm ]; then
simplegitam mali-r6p2-2016-add-mali450-sunxi-r7p0-platform.patch
simplegitam mali-r6p2-2017-modify-mali450-sunxi-r7p0-build.patch
simplegitam mali-r6p2-2018-fix-5.4-ump-arm-build.patch
fi

if [ -e drivers/gpu/arm ]; then
simplegitam mali-midgard-r14p0-fix_5.4_fence_debug.patch
fi

simplegitam linux-5.4.y-drm-panfrost-conflicts-with-midgard-and-bifrost.patch

#simplegitam linux-5.4.y-arm64-dts-rockchip-px30-add-gpu-nodes.patch
applypatches upstream series-upstream

applypatches upstream series-upstream-dsi

simplegitam linux-5.4.y-arm64-cpuinfo-add-system-serial-string.patch
simplegitam linux-5.4.y-nvmem-core-returning-bytes-read-is-successful-read-t.patch
simplegitam linux-5.4.y-soc-amlogic-add-cpuinfo-support.patch
simplegitam linux-5.4.y-arm64-dts-amlogic-ODROID-C2-add-cpuinfo.patch

simplegitam linux-5.4.y-drm-rockchip-Set-cur_ctr-to-0-for-clocks-between-150.patch
simplegitam linux-5.4.y-drm-rockchip-Add-162MHz-pixel-clock-for-1600x1200.patch
#simplegitam linux-5.4.y-drm-rockchip-Add-297MHz-pixel-clock-for-3840x2160.patch

git revert --no-edit ac7af4636327dbb2b3d1b8e3f84b0dfa8b4b4d8c
git revert --no-edit c0f69f98244ff42bf1a6f80e23f48959db8d3b1f
git revert --no-edit 08958ec3005e971555efe058c519102a5beebfb9


simplegitam linux-5.4.y-drm-rockchip-dw_hdmi-use-vendor-kernel-tables.patch
simplegitam d7527e920bca09b9059b6efc147b59a81eefea50.patch
simplegitam linux-5.4.y-clk-rockchip-rk3399-add-106.5MHz-clock-configuration.patch

simplegitam linux-5.4.y-clk-rockchip-add-new-pll-type-for-rk3366-and-similar.patch
simplegitam 894fca8f3cddc1364c4cfa25eadfa27ac1d1a9d8.patch
simplegitam a69364e01c76a445fb1078cd504289f0a45b0401.patch
simplegitam 5045fa1a3820cc314d9bb3b9ac9cd417ac2f1502.patch
simplegitam linux-5.4.y-clk-rockchip-fix-up-the-rockchip_rk3066_pll_clk_set_.patch

git revert --no-edit 96e88e683165227667c6fa0a6538af991a3f3674

simplegitam linux-5.4.y-arm-midgard-fix-merge.patch
simplegitam linux-5.4.y-fix-drm-exynos-backport-FBIOGET_DMABUF-from-3.10-ker.patch

simplegitam linux-5.4.y-drm-bridge-dw-hdmi-Add-Dynamic-Range-and-Mastering-I.patch
simplegitam 1bf95915a878b7041aa915558d76c579f61e2a0f.patch

simplegitam linux-5.4.y-clk-rockchip-pll-add-old_rate_and_grf.patch
simplegitam bd26d6793b7c5dde69dfc658f2a1db1248e44651.patch
simplegitam 6600132da6edfb67aeb76f2bad16e9da85cdd7bc.patch
simplegitam 810497ff787f9a01f59a993cda4308f3785d8f6a.patch
simplegitam linux-5.4.y-clk-rockchip-pll-remove-grf.patch

simplegitam e2229fb73e2120e7d107debd1aa4d153b255a2e2.patch
simplegitam c8ff64053757a71c9d963c091da0ef4cd517c9f9.patch

applypatches rk-5.4 series-rk-5.4-pre
applypatches rk-5.4-drm series-rk-5.4-drm
applypatches rk-5.4 series-rk-5.4-post

simplegitam 480ab0ee8b12d1da4c9ee72fd6facfb6d95b487b.patch

simplegitam mali-r6p2-2019-allow-continue-without-devfreq.patch
simplegitam mali-r6p2-2020-fix-device-tree-detection.patch
simplegitam mali-r6p2-2021-add-multi-platform-glue.patch

simplegitam rk-5.4/linux-5.4.y-drm-meson-dw-hdmi-wait-for-edid.patch

simplegitam mali-r6p2-2022-unify-resource-creation.patch
simplegitam mali-r6p2-2023-put-mali_platform_device_register-together.patch

simplegitam mali-r6p2-2024-arm-mali400-remove-TRACE_SYSTEM_STRING.patch

simplegitam upstream/d02d8a9ce80982374740b0a210723dbf0cdb66ce.patch
simplegitam upstream/de2501381470f7374cd502c239eecf8a3f67f65f.patch

simplegitam rk-5.4/linux-5.4.y-drm-rockchip-rk3066_hdmi-output-common-tv-resolution.patch
simplegitam rk-5.4/linux-5.4.y-drm-rockchip-rk3066_hdmi-add-bus_format.patch

# Commits on Aug 10, 2020
#simplegitam rk-5.4/8b3f2b0963d8c8141c4b5992e191ab2d9805aa5a.patch
#simplegitam rk-5.4/6a1ba17bc991fbc89202b12118f13d4ef9dfaf4a.patch
simplegitam rk-5.4/linux-5.4.y-drm-rockchip-add-rockchip_drm_add_modes_noedid.patch
simplegitam rk-5.4/linux-5.4.y-drm-rockchip-hdmi-use-rockchip_drm_add_modes_noedid-.patch
simplegitam rk-5.4/0a05fb7bcfa80dfed5dfcf11e7b4f3e624913c21.patch

# Commits on Aug 13, 2020
simplegitam rk-5.4/8d7e60f3192a3274e9a457c8868528da5fe5aa01.patch

# Commits on Aug 19, 2020
simplegitam rk-5.4/699c8ff6538644f1f198b55189f23036ac39b61d.patch

# Commits on Aug 27, 2020
simplegitam rk-5.4/655029432a973dc789aa5a7e5cb913b9f78e81d4.patch

simplegitam linux-5.4.y-soc-amlogic-meson-gx-cpuinfo-add-module-info.patch
simplegitam linux-5.4.y-soc-rockchip-cpuinfo-add-module-info.patch

simplegitam rk-5.4/linux-5.4.y-drm-bridge-synopsys-dsi-add-dual-dsi-support.patch
simplegitam rk-5.4/linux-5.4.y-drm-bridge-synopsys-dsi-add-power-on-off-optional-ph.patch
simplegitam rk-5.4/linux-5.4.y-drm-bridge-synopsys-dsi-driver-specific-configuratio.patch
simplegitam upstream/89bddff6be745fd688bbdd53773f5eb3136051eb.patch
simplegitam linux-5.4.y-drm-stm-dw_mipi_dsi-sync-with-kernel-changes.patch

simplegitam rk-5.4/linux-5.4.y-drm-bridge-synopsys-dsi-add-ability-to-have-glue-spe.patch
simplegitam rk-5.4/linux-5.4.y-drm-bridge-synopsys-dsi-Allow-VPG-to-be-enabled-via-.patch

simplegitam mali-midgard-r14p0-make-thirdparty-calls-optional.patch
simplegitam mali-midgard-r18p0-make-thirdparty-calls-optional.patch

#simplegitam rk-5.4/linux-5.4.y-drm-meson-dw_hdmi-adopt-format-handling-from-rockchi.patch
#simplegitam rk-5.4/linux-5.4.y-drm-meson-dw_hdmi-adopt-mode_valid-from-rockchip-dri.patch
simplegitam rk-5.4/linux-5.4.y-drm-bridge-synopsys-dw-hdmi-improve-ddc-error-handli.patch
fi

pcount=0
if [ -e patches/series ]; then
  pcount=`cat patches/series | grep -v -E '^#' | grep patch | wc -l`

  mv patches/series patches/series-git
fi

gcount=0
if [ -e patches/series-merge ]; then
  gcount=`cat patches/series-merge | grep -v -E '^#' | grep patch | wc -l`
fi
mcount=0
if [ -e patches/series-am ]; then
  mcount=`cat patches/series-am | grep -v -E '^#' | grep patch | wc -l`
fi
acount=0
if [ -e patches/series-no-am ]; then
  acount=`cat patches/series-no-am  | grep -v -E '^#' | grep patch | wc -l`
fi
ncount=0
if [ -e patches/series-no-git ]; then
  ncount=`cat patches/series-no-git | grep -v -E '^#' | grep patch | wc -l`
fi

echo ""

if [ $gcount -gt 0 -o $mcount -gt 0 -o $acount -gt 0 -o $ncount -gt 0 ]; then
  echo "Applied ${pcount} patches (${gcount} with git merge, ${mcount} with git am, ${acount} with git apply, ${ncount} outside git)."

else
  echo "Applied ${pcount} patches."
fi

echo ""

cfgmode=none
for f in ../patches/*_defconfig ; do
  c=`basename $f`
  c=${c/_defconfig/}

  if grep -q -E '^CONFIG_ARM=y|^CONFIG_ARM_.*=y' $f ; then
    if ! grep -q -E '^CONFIG_ARM64=y|^CONFIG_ARM64_.*=y' $f ; then
    #if [ -e arch/arm/configs/${c}_defconfig ]; then
    # mv arch/arm/configs/${c}_defconfig arch/arm/configs/${c}_git_defconfig
    #fi
    if [ -e arch/arm/configs/${c}_defconfig ]; then
      if ! diff -q $f arch/arm/configs/${c}_defconfig ; then
        cfgmode=update
      fi
    else
      if [ "${cfgmode}" = "none" ]; then
        cfgmode=add
      fi
    fi
    cp -p $f arch/arm/configs/
    fi
  fi
done

if [ "${cfgmode}" != "none" ]; then
  #LANG=C git diff arch/arm/configs
  git add arch/arm/configs
  git commit -m "arm: configs: $cfgmode sbc defconfigs"
fi

cfgmode=none
for f in ../patches/*_defconfig ; do
  c=`basename $f`
  c=${c/_defconfig/}

  if grep -q -E '^CONFIG_ARM64=y|^CONFIG_ARM64_.*=y' $f ; then
    #if [ -e arch/arm64/configs/${c}_defconfig ]; then
    # mv arch/arm64/configs/${c}_defconfig arch/arm64/configs/${c}_git_defconfig
    #fi
    if [ -e arch/arm64/configs/${c}_defconfig ]; then
      if ! diff -q $f arch/arm64/configs/${c}_defconfig ; then
        cfgmode=update
      fi
    else
      if [ "${cfgmode}" = "none" ]; then
        cfgmode=add
      fi
    fi
    cp -p $f arch/arm64/configs/
  fi
done

if [ "${cfgmode}" != "none" ]; then
  #LANG=C git diff arch/arm64/configs
  git add arch/arm64/configs
  git commit -m "arm64: configs: $cfgmode sbc defconfigs"
fi

cfgmode=none
for f in ../patches/*_defconfig ; do
  if [ -e $f ]; then
  c=`basename $f`
  c=${c/_defconfig/}

  if grep -q -E '^CONFIG_RISCV=y|^CONFIG_RISCV_.*=y' $f ; then
    #if [ -e arch/riscv/configs/${c}_defconfig ]; then
    # mv arch/riscv/configs/${c}_defconfig arch/riscv/configs/${c}_git_defconfig
    #fi
    if [ -e arch/riscv/configs/${c}_defconfig ]; then
      if ! diff -q $f arch/riscv/configs/${c}_defconfig ; then
        cfgmode=update
      fi
    else
      if [ "${cfgmode}" = "none" ]; then
        cfgmode=add
      fi
    fi
    cp -p $f arch/riscv/configs/
  fi
  fi
done

if [ "${cfgmode}" != "none" ]; then
  #LANG=C git diff arch/riscv/configs
  git add arch/riscv/configs
  git commit -m "riscv: configs: $cfgmode sbc defconfigs"
fi

echo OK
